/**
 * 网络请求
 */
import ext from './extend.js';
import tok from './token.js';

/**
 * @typedef {Object} RequestResult
 * @property {boolean} success - 请求是否成功标记
 * @property {string} message - 请求失败消息
 * @property {Object} data - 请求成功接收的数据
 */
const Request = {
    /**
     * 设置基础地址
     * @param {string} url - 基础地址
     */
    setBaseUrl(url) {
        if (!ext.isString(url)) {
            url = '';
        }
        ext.addStorage('base_url', url);
    },

    /**
     * 获取基础地址
     */
    getBaseUrl() {
        return ext.readStorage('base_url');
    },

    /**
     * 网络请求
     * @param {Object} opts - 请求参数
     * @param {string} opts.url - 请求地址
     * @param {string} opts.method - 请求方法，默认 get
     * @param {string} opts.dataType - 传输数据类型，默认 json
     * @param {string} opts.responseType - 接受数据类型，默认 json
     * @param {Object} opts.data - 请求参数
     * @param {Object} opts.header - 请求头
     * @returns {RequestResult} 网络请求结果对象
     */
    send(opts) {
        return new Promise(async (resolve, reject) => {
            if (!ext.isObject(opts)) {
                resolve({
                    success: false,
                    message: '参数不能为空'
                });
                return;
            }
            let data = null;
            let url = '';
            let baseUrl = Request.getBaseUrl();

            try {
                if (!ext.isUrl(opts.url)) {
                    if (ext.isEmptyString(baseUrl)) {
                        resolve({
                            success: false,
                            message: '未设置 BaseUrl'
                        });
                        return;
                    } else {
                        url = baseUrl + opts.url;
                    }
                }
                if (!ext.isObject(opts.header)) {
                    opts.header = {};
                }
                switch (opts.type) {
                    case 'json':
                        opts.header['content-type'] = 'application/json';
                        break;
                    case 'text':
                        opts.header['content-type'] = 'text/html; charset=utf-8';
                        break;
                    case 'form':
                        opts.header['content-type'] = 'multipart/form-data; boundary=something';
                        break;
                    case 'url':
                        opts.header['content-type'] = 'application/x-www-form-urlencoded';
                        break;
                }
                if (ext.isEmptyString(opts.method)) {
                    opts.method = 'get';
                }
                uni.request({
                    url: url,
                    method: opts.method || 'get',
                    dataType: opts.dataType || 'json',
                    responseType: opts.responseType || 'text',
                    header: opts.header || {},
                    data: opts.data,
                    timeout: 300000,
                    async success(res) {
                        if (res.statusCode === 401) {
                            let r = await tok.getByServer();
                            if (r.success) {
                                let rr = await Request.send(opts.data);
                                if (rr.success || rr.code === 200) {
                                    resolve({
                                        success: true,
                                        data: rr.data
                                    });
                                } else {
                                    resolve({
                                        success: false,
                                        message: rr.message
                                    });
                                }
                            } else {
                                resolve({
                                    success: false,
                                    message: r.message
                                });
                            }
                        }
                        resolve({
                            success: true,
                            data: res.data
                        });
                    },
                    fail(ex) {
                        console.error(ex);
                        ex = ext.errorFormatter(ex);
                        resolve({
                            success: false,
                            message: ex.message
                        });
                    }
                });
            } catch(ex) {
                let err = ext.errorFormatter(ex);
                console.error(ex);
                resolve({
                    success: false,
                    message: err.message
                });
            }
        });
    },

    /**
     * Token 网络请求
     * 在 header 中带 Authorization 参数。
     * @param {Object} opts - 请求参数
     * @param {string} opts.url - 请求地址
     * @param {string} opts.method - 请求方法，默认 get
     * @param {Object} opts.data - 请求参数
     * @param {Object} opts.header - 请求头
     * @returns {RequestResult} 网络请求结果对象
     */
    async sendToken(opts) {
        if (!ext.isObject(opts)) {
            return {
                success: false,
                message: '参数不能为空'
            };
        }
        let token = tok.get();
        if (ext.isNonEmptyString(token)) {
            opts.header = {
                Authorization: token
            };
        }
        return await this.send(opts);
    }
};

export default Request;